<?php

namespace engine;

use Exception;
use GMP;

class MySQLBitField
{
    private GMP $value;
    private int $size;

    public function __construct($value, int $size = 64) {
        $this->value = gmp_init($value);
        $this->size = $size;
    }

    /**
     * @throws Exception
     */
    public function has(int $bit): bool {
        $this->validateBit($bit);
        return gmp_testbit($this->value, $bit);
    }

    /**
     * @throws Exception
     */
    public function set(int $bit): void {
        $this->validateBit($bit);
        gmp_setbit($this->value, $bit);
    }

    /**
     * @throws Exception
     */
    public function clear(int $bit): void {
        $this->validateBit($bit);
        gmp_clrbit($this->value, $bit);
    }

    public function isEmpty(): bool {
        return !gmp_cmp($this->value, 0);
    }

    public function __toString(): string {
        $buf = '';
        for ($bit = $this->size - 1; $bit >= 0; --$bit)
            $buf .= gmp_testbit($this->value, $bit) ? '1' : '0';
        if (($pos = strpos($buf, '1')) !== false) {
            $buf = substr($buf, $pos);
        } else {
            $buf = '0';
        }
        return $buf;
    }

    /**
     * @throws Exception
     */
    private function validateBit(int $bit): void {
        if ($bit < 0 || $bit >= $this->size)
            throw new Exception('invalid bit ' . $bit . ', allowed range: [0..' . $this->size . ')');
    }
}